<?php
/*
	HashMask-512 : Secure Hash Encryption (Chiffrement par Hachage Sécurisé)
	Copyright © 2012 P.Lheureux (lheureux@free.fr) & D.Mestdagh (dimitri.mestdagh@laposte.net)
	
    This file is part of HashMask-512.
    HashMask-512 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published 
	by the Free Software Foundation, either version 3 of the License, 
	or (at your option) any later version.

    HashMask-512 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with HashMask-512. If not, see <http://www.gnu.org/licenses/>.
	
*/

class Hashmask {

	function __construct($debug=FALSE) {
		$this->debug = $debug;
		$this->nomFichierCleOut = 'hashmask.key';
		$this->nomFichierCleIn = 'key.dat';
		$this->nomFichierOut = 'datafile.hmk';
		if(is_file($this->nomFichierOut)){
			unlink($this->nomFichierOut);
		}		
	}
	
	/* Pour l'affichage des erreurs / exceptions : */
	public function afficheErreur($e){
		echo(utf8_decode($e)."<br />\n");
		exit;
	}
	
	/* Construction d'un masque du double de la longueur du texte à chiffrer : */
	private function creerMasque($reset=false){
		
		if(!isset($this->hasch) || $reset==true){
			$this->hasch = hash('sha512', $this->cleAleatoire.$this->cle);
		}
		
		for($i=0;$i<128;$i++){
			$this->hasch = hash('sha512', $this->hasch.$this->cle);
			$masque .= $this->hasch;
		}
		
		$this->masque = $masque;
		
		if($this->debug){
			echo("Masque : <br />");
			var_dump($masque);
			echo("<br /><br />");			
		}	
		
	}
	
	/* Masquage du texte : */
	private function masquer($chars){
		
		for($i=0;$i<strlen($chars);$i++){
			$j = $i*2;
			$c =  hexdec($this->masque[$j].$this->masque[$j+1]);
			$chars_masques .= chr( ord($chars[$i]) ^ $c );
		}
		
		return($chars_masques);
		
	}	
		
	/* Démasquage du texte : */
	private function demasquer($chars){
		
		for($i=0;$i<strlen($chars);$i++){
			$j = $i*2;
			$c =  hexdec($this->masque[$j].$this->masque[$j+1]);
			$chars_demasques .= chr($c ^ ord($chars[$i]));
		}
		
		return($chars_demasques);
		
	}
	
	/* Fonction pour retourner/inverser les caractères qui composent le nom du fichier : */
	private function envers($nom){
		for($i=strlen($nom);$i>0;$i--){
			$name .= $nom[$i-1];
		}
		return($name);
	}
	
	/* Chiffrement et déchiffrement : */
	public function cypher($nomFichierIn, $chiffrement){
	
		if(!move_uploaded_file($_FILES[$nomFichierIn]['tmp_name'], "./".$nomFichierIn)){
			/* Si l'upload n'a pas fonctionné : */
			$this->afficheErreur("Echec de l'envoi du fichier de données !");			  
		 }
	
		try {
			if (!is_readable($nomFichierIn)) {
				throw new Exception("Fichier <b>" . $_FILES[$nomFichierIn]['name'] . "</b> impossible à lire !");
			}		
		} catch (Exception $e) {
			$this->afficheErreur($e->getMessage());			
		}
		
		/* On crée le fichier à télécharger : */
		try {			
			@touch($this->nomFichierOut);
			if (!is_writable($this->nomFichierOut)) {
				throw new Exception("Fichier <b>" . $this->nomFichierOut . "</b> impossible à écrire !");
			}			
		} catch (Exception $e) {
			$this->afficheErreur($e->getMessage());
		}
		
		$handle = @fopen($nomFichierIn, "r");
		
		if($chiffrement){
			/* création d'une clé aléatoire : */
			for($i=0;$i<strlen($this->cle);$i++){
				$this->cleAleatoire .= sprintf("%c",rand(32,126));			
			}			
			$this->write2file($this->nomFichierOut,$this->cleAleatoire);
			/* on récupère le crc32 du fichier original : */
			$this->crc32original = sprintf("%010s",crc32(file_get_contents($nomFichierIn)));
			$name = $this->envers($_FILES[$nomFichierIn]['name']);			
			if($this->debug){
				echo("CRC32 du fichier original : <br />");
				var_dump($this->crc32original);
				echo("<br /><br />");			
			}			
		}else{
			$this->cleAleatoire = substr(file_get_contents($nomFichierIn), 0, strlen($this->cle));
		}
		
		$debut = microtime();
		
		if ($handle) {
			$segment = 0;
			while (!feof($handle)) {			  	
				$this->creerMasque();
				if($chiffrement){					
					if($segment==0){
						$contenu = fread($handle, 8182);
						$contenu = $this->crc32original.$contenu;
					}else{
						$contenu = fread($handle, 8192);
					}
					if(feof($handle)){
						/* on ajoute le nom du fichier original dans le fichier chiffré : */
						$contenu .= "*|".$name."|*";
					}
					$data = $this->masquer($contenu);
				}else{
					/* on extrait la clé aléatoire depuis le fichier chiffré : */
					if($segment==0){
						$contenu = substr(fread($handle, 8192+strlen($this->cle)), strlen($this->cle));
					}else{
						$contenu = fread($handle, 8192);
					}
					$data = $this->demasquer($contenu);
					if($segment==0){
						/* on extrait le CRC du fichier original depuis le fichier chiffré : */
						$this->crc32original = substr($data, 0, 10);						
						$data = substr($data, 10);
					}
					if(feof($handle)){
						/* on extrait le nom du fichier original depuis le fichier chiffré : */
						$intStart = strrpos($data, "*|");
						$intStop = strrpos($data, "|*");
						$length = $intStop - $intStart - 2;
						$name = preg_replace("/ /", "_", substr($data, $intStart+2, $length));
						$name = $this->envers($name);
						$data = substr($data, 0, $intStart).substr($data, $intStop+2);
						
					}					
				}
				/* on place les données traitées dans le fichier téléchargeable : */
				$this->write2file($this->nomFichierOut,$data);
				$segment=1;
			}
			if (!feof($handle)) {
				$this->afficheErreur("Erreur: échec lors de la lecture du fichier de données !");
			}
			fclose($handle);
			
		}
		
		$fin = microtime();
	   
		if($this->debug){	   
			/* on affiche le temps d'exécution (on multiplie par 1000 car on veut des millisecondes pas des micro-sec) : */
			$temps = ($fin - $debut)*1000;  /* vitesse d'AES (à battre ?) : 110 MB /sec */
			echo("Temps d'exécution pour le chiffrement/déchiffrement : ".number_format($temps, 2)." ms <br /><br />");
		}			
		
		unlink($nomFichierIn);
		
		/* en cas de déchiffrement : */
		if(!$chiffrement){
			
			/* on donne son nom au fichier en sortie (clair) : */
			if(!is_file($name)){
				@rename($this->nomFichierOut,$name);
				$this->nomFichierOut = $name;
			}else{
				$erreur = ("Fichier <b>" . $name . "</b> impossible à écrire !");
				$this->afficheErreur($erreur);
			}			
			
			/* on vérifie l'intégrité du fichier : */
			$verifcrc32 = sprintf("%010s",crc32(@file_get_contents($name)));
			if($this->debug){
				echo("CRC32 du fichier original : <br />");
				var_dump($this->crc32original);
				echo("<br /><br />");
				echo("CRC32 du fichier généré par le déchiffrement (à vérifier) : <br />");
				var_dump($verifcrc32);
				echo("<br /><br />");			
			}			
			if($verifcrc32 != $this->crc32original){				
				$erreur = ("Le fichier semble corrompu, ou vous n'utilisez pas la bonne clef !");
				@unlink($name);
				$this->afficheErreur($erreur);				
			}

		} 
		
		/* envoi du fichier demandé par l'utilisateur : */
		$this->getFile($this->nomFichierOut);
		
		return;
	
	}
	
	/* pour écrire dans le fichier en sortie : */
	private function write2file($file, $data){
				
		$fichier = fopen($file,'a'); 
		fwrite($fichier,$data); 
		fclose($fichier);
		
		return;
	
	}	
	
	/* Envoi du fichier à l'utilisateur : */
	private function getFile($nomFichier){
		if(!$this->debug){		
			/* Redirection de la page vers le téléchargement du fichier : */
			header("Content-disposition: attachment; filename=".$nomFichier);
			header("Content-Type: application/force-download");
			header("Content-Transfer-Encoding: text/plain\n");
			header("Content-Length: ".filesize($nomFichier));
			header("Pragma: no-cache");
			header("Cache-Control: must-revalidate, post-check=0, pre-check=0, public");
			header("Expires: 0");
		}else{
			echo("Contenu du fichier en sortie : <br />");
		}
		@readfile($nomFichier);								
		@unlink($nomFichier);
		exit;
	
	}
	
}

?>